//----------------------------------------//
// camera.h - header file for the camera class and functions, which
//			control the view of the scene that gets rendered
//
//----------------------------------------//

//include the other necessary files
#include "d3d.h"
//#include <stdio.h>	//file i/o stuff for camera configuration file


#ifndef _CAMERA_H_
#define _CAMERA_H_

//return types for the camera
const int CAMERA_OK		= 0;	
const int CAMERA_BAD	= 1;

enum CAMERAMODE
{
	FIRSTPERSON,	//first person shooter style camera
	THIRDPERSON,	//third person style camera
	STATIC			//camera doesn't move
};


class CAMERA
{
public:
	//constructor - gives default variables
	CAMERA();

	D3DXMATRIX matProjection;	//projection matrix
	float NearClip;				//near clip plane distance
	float FarClip;				//far clip plane distance
	float Fov;					//field of view

	D3DXMATRIX matView;			//view matrix
	D3DXVECTOR3 Position;		//camera position
	D3DXVECTOR3 Velocity;		//camera velocity
	D3DXVECTOR3 LookAt;			//lookat position
	float Yaw, YawVelocity;		//yaw rotation stuff
	float Pitch, PitchVelocity;	//pitch rotation stuff

	//int CameraState;

	//functions
	int InitializeCamera(LPDIRECT3DDEVICE8 device, int screenwidth, int screenheight);
	void TranslateCamera(float x, float y, float z);
	void RotateCamera(float yaw, float pitch);
	void UpdateCamera(LPDIRECT3DDEVICE8 device, float delta);

};





#endif //for _CAMERA_H_