//----------------------------------------//
// main.cpp - the main file for the windows stuff
//
//----------------------------------------//

//include the globals, as usual
#include <windows.h>
#include "engine.h"

//the global logfile
LOGFILE logfile;

//the global camera
CAMERA Camera;


//--------------------------------------------------//
// Name: MsgProc()
// Desc: The window's message handler
//--------------------------------------------------//
LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
        case WM_DESTROY:
            PostQuitMessage(0);
            return 0;

        case WM_PAINT:
            ValidateRect(hWnd, NULL);
            return 0;
    }

    return DefWindowProc(hWnd, msg, wParam, lParam);
}

//--------------------------------------------------//
// Name: WinMain()
// Desc: The application's main window loop function
//--------------------------------------------------//
INT WINAPI WinMain( HINSTANCE hinst, HINSTANCE, LPSTR, INT )
{

    WNDCLASS winclass;		//holds the window class thats created
	MSG msg;				//generic message
	bool notDone = true;	//flag for thread completion

	HWND hWnd;				//main window handle
	HINSTANCE hInst = hinst;	//handle to the instance of the window - used in DI

	//the big engine!!
	ENGINE engine;

	//first, fill in the window class structure
	winclass.style = CS_DBLCLKS ;
	winclass.lpfnWndProc = MsgProc;
	winclass.cbClsExtra = 0;
	winclass.cbWndExtra = 0;
	winclass.hInstance = hinst;

	//set the global to the same as this - it is needed for Direct Input
	//hInst = hinst;

	winclass.hIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON));
	winclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	winclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	winclass.lpszMenuName = NULL;
	winclass.lpszClassName = "Four Elements 3";

	//register the window class
	if(!RegisterClass(&winclass))
	return(0);

	//initialize the logfile first of all things
	if(logfile.CreateLogFile("log//logfile.log") != LOGFILE_OK)
	{
		MessageBox(NULL, "Failed to initialize the logfile.  Please make sure that all files are in the correct directories.", "Error!", MB_OK);
		return(0);	
	} else
		logfile.LogtoFile("----- Logfile Created -----\n");

	//read in the configuration file stuff so that the width and height of the window
	//are read in before the window is made
	if(engine.ReadConfigurationFile("system//config.cfg") != ENGINE_OK)
	{
		MessageBox(NULL, "Failed to read in configuration file.", "Error", MB_OK);
		logfile.LogtoFile("MAIN - engine.ReadConfigurationFile() failed.\n");
		return(0);
	} else
		logfile.LogtoFile("MAIN - engine.ReadConfigurationFile() successful.\n");


	//create the window
	if (!(hWnd = CreateWindow(winclass.lpszClassName, //class
			"Four Elements 3",			//title
			WS_OVERLAPPED | WS_SYSMENU,	//tags
			0,0,					//x,y position
			engine.d3d.ScreenWidth,		//window width
			engine.d3d.ScreenHeight,	// '   ' height
			NULL,					//handle to parent 
			NULL,					//handle to menu
			hinst,					//instance
			NULL)))					//creation params
		return(0);



	// Show the window
	ShowWindow(hWnd, SW_SHOWDEFAULT);
	UpdateWindow(hWnd);

	//initialize the engine and read in the configuration file
	if(engine.Initialize(hinst, hWnd) != ENGINE_OK)
	{
		MessageBox(NULL, "Failed to initialize program!", "Error", MB_OK);
		logfile.LogtoFile("MAIN - engine.Initialize() failed.\n");
		return(0);
	} else
		logfile.LogtoFile("MAIN - engine.Initialize() successful.\n");

	// Enter the message loop 
	while(notDone)
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			//test if it is a quit
			if(msg.message ==WM_QUIT) 
				break;

			//translate any accelerator keys 
			TranslateMessage(&msg);
			//send the message to the window proc
			DispatchMessage(&msg);

		} else {
			//run the main loop and see how it returns to see if the program needs
			//to quit or not
			HRESULT hr;
			hr = engine.MainLoop();
			if(hr == ENGINE_QUIT)
			{
				logfile.LogtoFile("ENG - Gamestate Set to Quit in main loop.  Quiting program... \n");
				notDone = false;
			}

			if(hr == ENGINE_BAD)
			{
				MessageBox(hWnd, "This program was unable to run.  If this problem persists, contact the proper people.", 
					"Critical Error", MB_OK);
				return 0;
			}
    	
		} //end if else for PeekMessage()

	} //end while 

	// Destroy the window
	DestroyWindow(hWnd);

	//undregister the windows class that was created
	UnregisterClass("Four Elements 3", winclass.hInstance);

	//close the logfile
	logfile.LogtoFile("----- Program Successful -----\n");
	logfile.CloseLogFile();

	//end the program
    return 0;
}

