//----------------------------------------//
// particle.cpp - main file for the particle system
//
//----------------------------------------//

//include the right stuff
#include "particle.h"

// the particle effect class stuff

//--------------------------------------------------//
// Name: ParticleEffect()
// Desc: constructor - gives default values
//--------------------------------------------------//
ParticleEffect::ParticleEffect()
{
	Array = NULL;
	Device = NULL;
	EmitterPosition = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	EmitterVelocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	FinalColor = 0x00000000;
	FinalSize = 0.0f;
	Gravity = D3DXVECTOR3(0.0f, -9.81f, 0.0f);
	InitialColor = 0x00000000;
	InitialSize = 0.0f;
	NumParticles = 0;
	ParticleVelocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	Spread = 0.0f;
	Texture = NULL;
	Type = 0;

}


//--------------------------------------------------//
// Name: CreateCustomEffect()
// Desc: creates a custom particle effect
//--------------------------------------------------//
int ParticleEffect::CreateCustomEffect(LPDIRECT3DDEVICE8 device, char *Filename)
{

	//do stuff here

	//return goood
	return PARTICLE_OK;
}

//--------------------------------------------------//
// Name: CreateEffect()
// Desc: creates a pre-built effect
//--------------------------------------------------//
int ParticleEffect::CreateEffect(LPDIRECT3DDEVICE8 device, int type, D3DXVECTOR3 pos, D3DXVECTOR3 direction)
{
	//get whats in the parameters
	Device = device;
	Type = type;

	//figure out what type of effect it is first
	switch(Type)
	{
		case FIRE:
			//since its fire, set up the default variables...
				EmitterPosition = pos;
				EmitterVelocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
				FinalColor = 0x00ff0000;
				FinalSize = 0.0f;
				Gravity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
				InitialColor = 0x00ffff00;
				InitialSize = 0.10f;
				NumParticles = 100;
				ParticleVelocity = D3DXVECTOR3(0.0f, 2.50f, 0.0f);
				Spread = 0.0f;

				//load the particle texture
				if(FAILED(D3DXCreateTextureFromFileExA(Device,	//d3d device 
							"data\\textures\\particle.bmp",		//filename
							D3DX_DEFAULT,	//width
							D3DX_DEFAULT,	//height
							D3DX_DEFAULT,	//mip levels
							0,				//usage - not a render target
							D3DFMT_UNKNOWN,	//format
							D3DPOOL_DEFAULT, //memory pool
							D3DX_DEFAULT, //filtering
							D3DX_DEFAULT, //mipmap filtering
							NULL,		//colorkey
							NULL,		//source info
							NULL,		//palette
							&Texture)))
				{
					//looks like it failed, which is a bad thing
					logfile.LogtoFile("PAR - failed to load particle system fire texture.\n");
					return PARTICLE_BAD;
				}
				
				
				break;
		case SMOKEWAFT:
				

				break;
		case SMOKETRAIL:
				

				break;
	} //end switch


	//create the new array of particles and check to see if it was made ok
	Array = new Particle[NumParticles];

	//check if it made things alright
	if(!Array)
	{
		return PARTICLE_BAD;
	}

	//create the vertex buffer
    if(FAILED(device->CreateVertexBuffer(4*sizeof(PARTICLEVERTEX)*NumParticles,
									D3DUSAGE_WRITEONLY, D3DFVF_PARTICLEVERTEX,
									D3DPOOL_DEFAULT, &VertexBuffer)))
	{
		return PARTICLE_BAD;
	}

	//now loop through the particles and give them some starting values
	for(int i=0; i<NumParticles; i++)
	{
		//set the flags so they aren't all drawn by default
		Array[i].Energy = 0;
		Array[i].Draw = DONTDRAW;
		Array[i].Color = 0x00000000;
		Array[i].Position = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
		Array[i].Velocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
		Array[i].Size = 0.0f;
	}

	//return ok
	return PARTICLE_OK;
}

//--------------------------------------------------//
// Name: DestroyEffect()
// Desc: de-allocates all the stuff for the effect
//--------------------------------------------------//
void ParticleEffect::DestroyEffect()
{
	//get rid of the vertex buffer and the array of particles
	delete [] Array;
	SAFE_RELEASE(VertexBuffer);


}


//--------------------------------------------------//
// Name:UpdateEffect()
// Desc: updates the positions of the particles
//--------------------------------------------------//
int ParticleEffect::UpdateEffect()
{
	//update the specific effects in specific ways
	switch(Type)
	{
		case FIRE:
			{
				//loop through all the particles and lower thier energy.  If they
				//are at 0, tag them so that they aren't drawn;
				for(int i =0; i<NumParticles; i++)
				{
					if(Array[i].Energy > 0)
					{
						//lower the energy
						Array[i].Energy--;

						//move the particles
						//Array[i].Position +=(ParticleVelocity
					}

					if(Array[i].Energy <= 0)
					{
						Array[i].Energy = 0;
						Array[i].Draw = DONTDRAW;
					}

				}

				//find a new particle to revive
				for(i = 0; i<NumParticles; i++)
				{
					if(Array[i].Draw == DONTDRAW)
						break;

				}

				Array[i].Color = InitialColor;
				Array[i].Draw = DRAW;
				Array[i].Energy = 100;
				Array[i].Position = EmitterPosition;
				Array[i].Size = InitialSize;
				Array[i].Velocity = ParticleVelocity;

			}
			break;
		case SMOKEWAFT:
			//put code here
			break;
		case SMOKETRAIL:
			//put code here as well
			break;
	} //end switch



	//went ok...
	return PARTICLE_OK;
}


//--------------------------------------------------//
// Name: RenderEffect()
// Desc: renders the effect
//--------------------------------------------------//
int ParticleEffect::RenderEffect()
{


	//must have gone ok...
	return PARTICLE_OK;
}
