//----------------------------------------//
// particle.h - header file for the particle system
//
//----------------------------------------//

//include the other stuff that should be included
#include "d3d.h"
#include "enginedef.h"

//return types for the particle system
const int PARTICLE_OK	= 0;
const int PARTICLE_BAD	= 1;
const int DRAW		= 10;	//draw the particle	
const int DONTDRAW	= 11;	//don't draw the particle

//enum for the particle effect shapes
enum EFFECTSHAPE
{
	EMITTER_POINT = 0,
	EMITTER_SPHERE,		//not yet implemented
	EMITTER_PLANE		//not yet implemented
};

//TODO - finish all this stuff
enum EFFECTTYPE
{
	FIRE	= 0,
	SMOKEWAFT,
	SMOKETRAIL
};


//the particle struct
struct PARTICLEVERTEX
{
    D3DXVECTOR3 position;
    DWORD color;
	float tu, tv;		// The texture coordinates
};

//custom 3d vertex definition
#define D3DFVF_PARTICLEVERTEX (D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX0)


//the particle class
class Particle
{
public:
	PARTICLEVERTEX Vertices[5];
	D3DXVECTOR3 Position;
	//*****D3DXVECTOR3 PreviousPosition;
	D3DXVECTOR3 Velocity;
	DWORD Color;
	int Energy;
	int Draw;
	float Size;

};

//the particle effect class
class ParticleEffect
{
public:
	//con/destructor
	ParticleEffect();

	//variables
	LPDIRECT3DDEVICE8 Device;	//the d3d device
	LPDIRECT3DTEXTURE8 Texture;	//the texture for all the particles
	LPDIRECT3DVERTEXBUFFER8 VertexBuffer;
	Particle *Array;			//pointer to an array of particles
	int NumParticles;			//total number of particles in the effect

	//attributes...
	int Type;		//the type of particle effect
	D3DXVECTOR3 EmitterPosition;	//position of the emitter
	D3DXVECTOR3 EmitterVelocity;	//velocity of the emitter
	D3DXVECTOR3 ParticleVelocity;	//maximum particle velocity
	D3DXVECTOR3 Gravity;	//gravity.  what else?
	DWORD InitialColor;		//initial color of the particle
	DWORD FinalColor;		//final color of the particle
	float Spread;		//how much the particle spread when they are emitted
	float InitialSize;	//initial size of the particle
	float FinalSize;	//final size of the particle

	//functions...
	int CreateCustomEffect(LPDIRECT3DDEVICE8 device, char *Filename);
	int CreateEffect(LPDIRECT3DDEVICE8 device, int type, D3DXVECTOR3 pos, 
										D3DXVECTOR3 direction);
	int UpdateEffect();
	int RenderEffect();
	void DestroyEffect();

};