//----------------------------------------//
// timer.cpp - main file for timer class/functions
//
//----------------------------------------//

//include the global variables, if necessary
#include "timer.h"


//--------------------------------------------------//
// Name: InitTimer()
// Desc: function that sets up the timer to 1/30 of a second
//--------------------------------------------------//
int TIMER::InitTimer()
{
	//set some default values first
	NextTime = 0;
	PreviousTime = 0;
	Delta = 0.0f;
	Counter = 0;
	
	//check for a performance counter
	if(QueryPerformanceFrequency((LARGE_INTEGER *)&PerformanceFrequency))
	{
		//it got the performance frequency, so continue
		TimeCount = (long)PerformanceFrequency/30;
		QueryPerformanceCounter((LARGE_INTEGER *)&NextTime);
		TimeScale = 1.0/PerformanceFrequency;

	} else {
		// it failed
		return TIMER_BAD;
	}

	//save the time of the last frame
	PreviousTime = NextTime;

	//return ok
	return TIMER_OK;
}

//--------------------------------------------------//
// Name: CheckOneSecondTimer()
// Desc: the function that returns yes every 1 second
//--------------------------------------------------//
int TIMER::CheckOneSecondTimer()
{
	//check to see if it has fired 30 times.  If it has, do the right return 
	//thing.  Counter is static, so it won't be set to 0 every time the function
	//is called
	if(Counter >= 30)
	{
		Counter = 0;
		return TIMER_OK;
	} else {
		return NOT_YET;
	}

}

//--------------------------------------------------//
// Name: CheckTimer()
// Desc: function that checks the time to see if it 
//		is time to return yes
//--------------------------------------------------//
int TIMER::CheckTimer()
{
	//check to see if it is time yet
	if(CurrentTime >= NextTime)
	{
		//save the frame time
		PreviousTime = CurrentTime;

		//set the next time for the timer to fire
		NextTime = CurrentTime + TimeCount;

		//increment the counter to say that 1/30th of a second has passed
		Counter++;

		return TIMER_OK;
	} else {
		//it isn't time yet
		return TIMER_BAD;
	}

}


//--------------------------------------------------//
// Name: UpdateTimer()
// Desc: function that updates the timer so that timeGetTime 
//		doesn't have to be called so many times
//--------------------------------------------------//
void TIMER::UpdateTimer()
{
	//calculate the new Delta value
	Delta = (float)((CurrentTime-PreviousTime)*TimeScale);

	//now update the previous time
	PreviousTime = CurrentTime;

	//update the current time
	QueryPerformanceCounter((LARGE_INTEGER *)&CurrentTime);

}